"""Классы. Окно статистики сообщений"""
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QTableView, QDialog, QPushButton
from PyQt5.QtCore import Qt


class HistoryWindow(QDialog):
    """
    Класс. Окно с историей пользователей
    """
    def __init__(self, database):
        super().__init__()
        self.database = database
        self.init_ui()

    def init_ui(self):
        """
        Дополнительная инициализация окна со статистикой сообщений
        :return:
        """
        # Настройки окна:
        self.setWindowTitle('Статистика клиентов')
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        # Кнапка закрытия окна
        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        # Лист с собственно историей
        self.history_table = QTableView(self)
        self.history_table.move(10, 10)
        self.history_table.setFixedSize(580, 620)

        self.create_stat_model()
        # self.show()

    def create_stat_model(self):
        """ Метод - реализует заполнение таблицы историей сообщений """

        # Список записей из БД
        stat_list = self.database.message_history()

        # объект модели данных
        q_ls = QStandardItemModel()
        q_ls.setHorizontalHeaderLabels(
            ["Имя клиента",
             "Последний взод",
             "Сообщений отправлено",
             "Сообщений получено"]
        )

        for row in stat_list:
            user, last_seen, sent, recvd = row
            user = QStandardItem(user)
            user.setEditable(False)
            last_seen = QStandardItem(str(last_seen.replace(microsecond=0)))
            last_seen.setEditable(False)
            sent = QStandardItem(str(sent))
            sent.setEditable(False)
            recvd = QStandardItem(str(recvd))
            recvd.setEditable(False)
            q_ls.appendRow([user, last_seen, sent, recvd])

        self.history_table.setModel(q_ls)
        self.history_table.resizeColumnsToContents()
        self.history_table.resizeRowsToContents()



"""Тестовый модуль"""
import sys
import unittest
# from common.errors import ReqFieldMissingError, ServerError
# from common.variables import *
# from client import create_presence, process_response_ans
sys.path.append('../')


class TestClass(unittest.TestCase):
    """# Класс с тестами"""
    pass
    # тест коректного запроса
    # def test_def_presense(self):
    #     test = create_presence('Guest')
    #     # время необходимо приравнять принудительно иначе тест никогда не
    #     будет
    #     # пройден
    #     test[TIME] = 1.1
    #     self.assertEqual(
    #         test, {
    #             ACTION: PRESENCE, TIME: 1.1, USER: {
    #                 ACCOUNT_NAME: 'Guest'}})

    # # тест корректтного разбора ответа 200
    # def test_200_ans(self):
    #     self.assertEqual(process_response_ans({RESPONSE: 200}), '200 : OK')
    #
    # # тест корректного разбора 400
    # def test_400_ans(self):
    #     self.assertRaises(
    #         ServerError, process_response_ans, {
    #             RESPONSE: 400, ERROR: 'Bad Request'})
    #
    # # тест исключения без поля RESPONSE
    # def test_no_response(self):
    #     self.assertRaises(
    #         ReqFieldMissingError, process_response_ans, {
    #             ERROR: 'Bad Request'})


if __name__ == '__main__':
    unittest.main()



"""Файл дескрипторов"""
import ipaddress
import logging
import sys

# Инициализиция логера
# метод определения модуля, источника запуска.
if sys.argv[0].find('client') == -1:
    # если не клиент то сервер!
    logger = logging.getLogger('server')
else:
    # ну, раз не сервер, то клиент
    logger = logging.getLogger('client')


class Port:
    """
    Класс - Дескриптор для описания порта:
    """
    def __set__(self, instance, value):
        if not 1023 < value < 65536:
            logger.critical(
                f'Попытка запуска с указанием неподходящего порта {value}. '
                f'Допустимы адреса с 1024 до 65535.')
            sys.exit(1)
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


class Ip:
    """
    Класс - верификации ip-адреса
    """
    def __set_name__(self, owner, name):
        self.name = name

    def __set__(self, instance, value):
        if value == '':
            instance.__dict__[self.name] = value
            return

        # проверка ip-адреса
        try:
            ip = ipaddress.ip_address(value)
            instance.__dict__[self.name] = str(ip)
        except Exception:
            raise ValueError(f"IP address {value} is not valid")


import sys
from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QApplication, QLabel, \
    QTableView, QDialog, QPushButton, QLineEdit, QFileDialog, QMessageBox
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import Qt
import os



def gui_create_model(database):
    """
    GUI - Создание таблицы QModel, для отображения в окне программы.
    :param database:
    :return:
    """
    list_users = database.active_users_list()
    list = QStandardItemModel()
    list.setHorizontalHeaderLabels(
        ['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
    for row in list_users:
        user, ip, port, time = row
        user = QStandardItem(user)
        user.setEditable(False)
        ip = QStandardItem(ip)
        ip.setEditable(False)
        port = QStandardItem(str(port))
        port.setEditable(False)
        # Уберём милисекунды из строки времени, т.к. такая точность не
        # требуется.
        time = QStandardItem(str(time.replace(microsecond=0)))
        time.setEditable(False)
        list.appendRow([user, ip, port, time])
    return list



def create_stat_model(database):
    """
    # GUI - Функция реализующая заполнение таблицы историей сообщений.
    :param database:
    :return:
    """
    # Список записей из базы
    hist_list = database.message_history()

    # Объект модели данных:
    list = QStandardItemModel()
    list.setHorizontalHeaderLabels(
        ['Имя Клиента', 'Последний раз входил', 'Сообщений отправлено', 'Сообщений получено'])
    for row in hist_list:
        user, last_seen, sent, recvd = row
        user = QStandardItem(user)
        user.setEditable(False)
        last_seen = QStandardItem(str(last_seen.replace(microsecond=0)))
        last_seen.setEditable(False)
        sent = QStandardItem(str(sent))
        sent.setEditable(False)
        recvd = QStandardItem(str(recvd))
        recvd.setEditable(False)
        list.appendRow([user, last_seen, sent, recvd])
    return list


if __name__ == '__main__':
    '''
    app = QApplication(sys.argv)
    ex = MainWindow()
    ex.statusBar().showMessage('Test Statusbar Message')
    test_list = QStandardItemModel(ex)
    test_list.setHorizontalHeaderLabels(['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
    test_list.appendRow([QStandardItem('1'), QStandardItem('2'), QStandardItem('3')])
    test_list.appendRow([QStandardItem('4'), QStandardItem('5'), QStandardItem('6')])
    ex.active_clients_table.setModel(test_list)
    ex.active_clients_table.resizeColumnsToContents()
    print('JKJKJK')
    app.exec_()
    print('END')'''
    app = QApplication(sys.argv)
    message = QMessageBox
    # dial = ConfigWindow()

    app.exec_()



"""Файл сервера"""
import socket
import os
import argparse
import select
import sys
import threading
import configparser
from PyQt5.QtWidgets import QApplication, QMessageBox
from PyQt5.QtCore import QTimer, Qt

from common.utils import *
from common.decos import log
from common.variables import DESTINATION
from common.descryptors import Port
from common.metaclasses import ServerMaker
from server.core import MessageProcessor
from server.database import ServerStorage
from server.server_gui import gui_create_model, create_stat_model
from server.config_window import ConfigWindow
from server.main_window import MainWindow
from server.history_window import HistoryWindow

# Инициализация логирования сервера.
logger = logging.getLogger('server')

# Флаг что был подключён новый пользователь, нужен чтобы не мучать BD
# постоянными запросами на обновление
new_connection = False
conflag_lock = threading.Lock()



@log
def arg_parser(default_port, default_address):
    """
    Парсер аргументов коммандной строки.
    :param default_port:
    :param default_address:
    :return:
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', default=default_port, type=int, nargs='?')
    parser.add_argument('-a', default=default_address, nargs='?')
    parser.add_argument('--no_gui', action='store_true')
    namespace = parser.parse_args(sys.argv[1:])
    listen_address = namespace.a
    listen_port = namespace.p
    gui_flag = namespace.no_gui
    return listen_address, listen_port, gui_flag


def config_load():
    """
    Загрузка файла конфигурации
    :return:
    """
    config = configparser.ConfigParser()
    dir_path = os.getcwd()
    config.read(f"{dir_path}/{'server.ini'}")
    # Если конфиг файл загружен правильно, запускаемся, иначе конфиг по
    # умолчанию.
    if 'SETTINGS' in config:
        return config
    else:
        config.add_section('SETTINGS')
        config.set('SETTINGS', 'Default_port', str(DEFAULT_PORT))
        config.set('SETTINGS', 'Listen_Address', '')
        config.set('SETTINGS', 'Database_path', '')
        config.set('SETTINGS', 'Database_file', 'server_database.db3')
        return config


def main():
    """
    Основная функция main()
    :return:
    """
    # Загрузка файла конфигурации сервера
    config = config_load()

    # Загрузка параметров командной строки, если нет параметров, то задаём
    # значения по умоланию.
    listen_address, listen_port, gui_flag = arg_parser(config['SETTINGS']['Default_port'],
                                                       config['SETTINGS']['Listen_Address'])

    # Инициализация базы данных
    database = ServerStorage(os.path.join(
        config['SETTINGS']['Database_path'],
        config['SETTINGS']['Database_file']))

    # Создание экземпляра класса - сервера и его запуск:
    server = MessageProcessor(listen_address, listen_port, database)
    server.daemon = True
    server.start()

    # Переключатель GUI
    if gui_flag:
        while True:
            command = input('Нажмите 9 для завершения работы сервера')
            if command == '9':
                server.running = False
                server.join()
                break
    else:
        # графическое окружение для сервера
        server_app = QApplication(sys.argv)
        server_app.setAttribute(Qt.AA_DisableWindowContextHelpButton)
        main_window = MainWindow(database, server, config)

        # запуск GUI
        server_app.exec_()

        # при закрытии окон останавливаем обработчик сообщений
        server.running = False


if __name__ == '__main__':
    main()
